#include <msp/gl/meshbuilder.h>
#include "image.h"
#include "part.h"
#include "style.h"

using namespace std;

namespace Msp {
namespace GLtk {

Image::Image(const GL::Texture2D *i):
	image(i),
	keep_aspect(true)
{
	focusable = false;
}

void Image::autosize_special(const Part &part, Geometry &ageom) const
{
	if(part.get_name()=="image")
	{
		const Sides &margin = part.get_margin();
		if(image)
		{
			ageom.w = max(ageom.w, image->get_width()+margin.left+margin.right);
			ageom.h = max(ageom.h, image->get_height()+margin.top+margin.bottom);
		}
		else
		{
			ageom.w = max(ageom.w, margin.left+margin.right);
			ageom.h = max(ageom.h, margin.top+margin.bottom);
		}
	}
}

void Image::set_image(const GL::Texture2D *i)
{
	image = i;
	signal_autosize_changed.emit();
	rebuild();
}

void Image::set_keep_aspect(bool ka)
{
	keep_aspect = ka;
	rebuild();
}

void Image::rebuild_special(const Part &part)
{
	if(part.get_name()=="image")
	{
		if(!image)
			return;

		const Alignment &align = part.get_alignment();
		Geometry rgeom = part.get_geometry();
		align.apply(rgeom, geom, part.get_margin());

		if(keep_aspect)
		{
			float aspect = static_cast<float>(image->get_width())/image->get_height();
			if(rgeom.w<rgeom.h*aspect)
			{
				unsigned h = static_cast<unsigned>(rgeom.w/aspect);
				rgeom.y += (rgeom.h-h)*align.y;
				rgeom.h = h;
			}
			else
			{
				unsigned w = static_cast<unsigned>(rgeom.h*aspect);
				rgeom.x += (rgeom.w-w)*align.x;
				rgeom.w = w;
			}
		}

		GL::MeshBuilder bld(part_cache.create_mesh(part, *image));
		bld.color(1.0f, 1.0f, 1.0f);
		bld.begin(GL::QUADS);
		bld.texcoord(0.0, 0.0);
		bld.vertex(rgeom.x, rgeom.y);
		bld.texcoord(1.0, 0.0);
		bld.vertex(rgeom.x+rgeom.w, rgeom.y);
		bld.texcoord(1.0, 1.0);
		bld.vertex(rgeom.x+rgeom.w, rgeom.y+rgeom.h);
		bld.texcoord(0.0, 1.0);
		bld.vertex(rgeom.x, rgeom.y+rgeom.h);
		bld.end();
	}
}

} // namespace GLtk
} // namespace Msp
