#include <msp/fs/utils.h>
#include "resources.h"

using namespace std;

namespace Msp {
namespace GLtk {

Resources::Resources()
{
	init();
}

Resources::Resources(const FS::Path &fn)
{
	init();

	dir_src = new DataFile::DirectorySource;
	dir_src->add_directory(FS::dirname(fn));
	add_source(*dir_src);

	DataFile::load(*this, fn.str());
}

void Resources::init()
{
	default_font = 0;
	dir_src = 0;
	add_type<Graphic>().keyword("graphic");
	add_type<GL::Texture2D>().keyword("texture").creator(&Resources::create_texture);
	add_type<GL::Font>().keyword("font");
	add_type<Style>().keyword("style");
}

Resources::~Resources()
{
	delete dir_src;
}

const GL::Font &Resources::get_default_font() const
{
	if(!default_font)
		throw logic_error("!default_font");

	return *default_font;
}

GL::Texture2D *Resources::create_texture(const string &name)
{
	string ext = FS::extpart(name);
	if(ext==".png" || ext==".jpg")
		if(IO::Seekable *io = open_from_sources(name))
		{
			Graphics::Image image;
			image.load_io(*io);
			delete io;

			GL::Texture2D *tex = new GL::Texture2D;
			tex->set_min_filter(GL::LINEAR);
			tex->image(image);
			return tex;
		}

	return 0;
}


Resources::Loader::Loader(Resources &r):
	Collection::Loader(r),
	res(r)
{
	add("default_font", &Loader::default_font);
	add("font", &Loader::font);
}

void Resources::Loader::default_font(const string &name)
{
	res.default_font = &res.get<GL::Font>(name);
}

void Resources::Loader::font(const string &name)
{
	RefPtr<GL::Font> fnt = new GL::Font;
	load_sub(*fnt, res);
	res.add(name, fnt.get());
	if(!res.default_font)
		res.default_font = fnt.get();
	fnt.release();
}

} // namespace GLtk
} // namespace Msp
